
#include <cstdio>
#include <cstring>
#include <cmath>
#include <vector>

#include "gr.h"



using namespace std;

#include "Graph.h"
#include "GraphView.h"


CGraphView::CGraphView(void)
{
    pGraph = NULL;
    penDownFlag = false;
    lineDrawn = false;
    m_bMultiColor = true;
    penColor = GC_RED;
}

CGraphView::~CGraphView()
{
}


void CGraphView::drawLine(int x0, int y0, int x1, int y1)
{
    glBegin(GL_LINES);
    glVertex2i(x0, y0);
    glVertex2i(x1, y1);
    glEnd();
}


// //////////////////// utilities /////////////////////
//
//   scaling routines for display 
//

int CGraphView::cXScale( double x )
{
    return  cXOffset + (int)(cXScaleFactor*(x-rXMin)+0.5) ;
}

int CGraphView::cYScale( double y)
{
    return  cYOffset - (int)(cYScaleFactor*(y-rYMin)+0.5) ;
}

double CGraphView::cXScaleBack( int ix)
{
    return  rXMin + (ix-cXOffset)/cXScaleFactor;
}

int CGraphView::cXScaleBackInt(int ix)
{
    double xt;
    int ixt;
    
    xt = cXScaleBack(ix);  
    ixt = (int)((xt - pGraph->XOrigin)/(pGraph->XDelta));
    
    if(ixt<0)ixt = 0;
    if(ixt>pGraph->npts)ixt =pGraph->npts-1;
	
    if(pGraph->W[pGraph->XAxisNo].A[ixt]<xt){
        while(pGraph->W[pGraph->XAxisNo].A[ixt]<xt && ixt < pGraph->npts-1){
            ixt++;
        }
    }
    
    if(pGraph->W[pGraph->XAxisNo].A[ixt]>xt){
        while(pGraph->W[pGraph->XAxisNo].A[ixt]<xt && ixt>0){
            ixt--;
        }
    }
	
    if(ixt<0)ixt = 0;
    if(ixt>pGraph->npts)ixt =pGraph->npts-1;
	
    return ixt;
}

double CGraphView::cYScaleBack(int iy)
{
    return  rYMin + (cYOffset -iy)/cYScaleFactor;
}


// =====  Tic routines ===== 

double CGraphView::autoRange(double ZMin,double ZMax)
{
    int nTic = 3;
    double ZDelta, ZTic, Zn;
    
    ZDelta = ZMax-ZMin;
    Zn = (int)log10(ZDelta);
    ZTic = pow(10.0, Zn);
    while(1){
        if ( (ZDelta/ZTic) >= nTic)break;
        
        ZTic = 0.5 *  ZTic;
        if((ZDelta/ZTic)>= nTic)break;
        
        ZTic = 0.4 *  ZTic;
        if((ZDelta/ZTic)>=nTic)break;
        
        ZTic = 0.5 *  ZTic;
        if((ZDelta/ZTic)>=nTic)break;
    }
    return ZTic;
}




void CGraphView::setPenColor(int cl)
{
    switch (cl){
        case GC_GRAY: // gray
            glColor3f(0.5, 0.5, 0.5);
            break;
        case GC_RED: // red
            glColor3f(1.0, 0.0, 0.0);
            break;
        case GC_GREEN: // green
            glColor3f(0.0, 0.5, 0.0);
            break;
        case GC_BLUE: // blue
            glColor3f(0.0, 0.0, 1.0);
            break;
        case GC_ORANGE: // orange
            glColor3f(1.0, 0.5, 0.0);
            break;
        case GC_DARKBLUE: // dark blue
            glColor3f(0.0, 0.0, 0.5);
            break;
        case GC_YELLOW: // yellow
            glColor3f(1.0, 1.0, 0.0);
            break;
        case GC_PURPLE: // purple
            glColor3f(0.5, 0.0, 1.0);
            break;
        default:
            glColor3f(0.0, 0.0, 0.0);
    }
}

//===== Drawing Graph ===== 
//
//     Draw a graph using information stored in aGraph

void CGraphView::drawAxis(int width, int height)
{
    int TicLength;
    double CmToPixels=38.0;
    double TicUnit;
    
    int i, n;
    int cX, cY, dcX;
    double aX, aY;
    char t[20];
    char* s;
    
    
    rXMin = pGraph->XMin;
    rXMax = pGraph->XMax;
    
    rYMin = pGraph->YMin;
    rYMax = pGraph->YMax;
    
    if(pGraph->XMin == pGraph->XMax)pGraph->XMax = pGraph->XMin + 1.0;
    if(rYMin == rYMax) rYMax = rYMin + 1.0;
    
    // margin: 2.0 cm for left and bottom, 1.0 cm for right and top  
    cXOffset = (int)(CmToPixels * 2.0);
    cYOffset = (int)(height-CmToPixels * 1.5);
    
    cXScaleFactor = (width-CmToPixels* 3.0) /(pGraph->XMax-pGraph->XMin); 
    cYScaleFactor = (height-CmToPixels* 2.5) /(rYMax-rYMin);
    TicLength = (int)(0.15 * CmToPixels);
    
    // X-Axis 
    TicUnit = autoRange(pGraph->XMin, pGraph->XMax);
    n = (int)( -log10(TicUnit)+1 );
    if( n < 0 ) n = 0;	
    
    cX = cXScale(pGraph->XMax);
    drawLine(cXOffset, cYOffset, cX, cYOffset);
    
    i= (int)(rXMin / TicUnit);
    if( i*TicUnit < rXMin ) i++;
    while(1){
        aX = i*TicUnit;
        if(aX > pGraph->XMax ) break;
        sprintf(t,"%.*f",n,aX);
        cX = cXScale(aX);
        dcX = strlen(t) * 8/2;
        glRasterPos2d(cX-dcX,cYOffset+TicLength+13);
        s = t;
        while(*s){
            glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *s);
            ++s;
        }
        drawLine(cX, cYOffset, cX, cYOffset+TicLength);
        i++;
    }
    
    // Y-Axis 
    cY = cYScale(rYMax);
    drawLine(cXOffset, cYOffset, cXOffset, cY);
    
    TicUnit = autoRange(rYMin,rYMax);
    n = (int)( -log10(TicUnit)) +1;
    if ( n< 0 ) n= 0;
    i= (int)(rYMin / TicUnit);
    if(i*TicUnit < rYMin)i++;
    
    while(1){
        aY = i*TicUnit;
        if(aY > rYMax)break;
        
        sprintf(t,"%.*f",n,aY);
        cY = cYScale(aY);
        dcX = strlen(t)*8 + 2*TicLength;
        glRasterPos2d(cXOffset-dcX, cY + 4);
        s = t;
        while(*s){
            glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *s);
            ++s;
        }
        drawLine(cXOffset-TicLength, cY,  cXOffset, cY);
        i++;
    }
}



void CGraphView::drawTrace()
{
    int NewX, NewY;
    int i,j;
    int OldX = -1000;
    int OldY = -1000;
    double ax,ay;
    
    double* px = &(pGraph->W[pGraph->XAxisNo].A[0]);
    
    for(i = 1 ; i < pGraph->nwaves ; i++){  // wave 0 is x-axis
        
        
        if(pGraph->W[i].visible && i!=pGraph->XAxisNo){
            
            if(m_bMultiColor){
                setPenColor(i % 8);
            }else{
                setPenColor(penColor);
            }
            
            glBegin(GL_LINE_STRIP);
            
            double* py = &(pGraph->W[i].A[0]);
            for(j = pGraph->XPStart ; j < pGraph->XPStart +pGraph->Points ; j++){
                
                ax = px[j];
                if(ax>rXMax)ax=rXMax;else if(ax<rXMin)ax=rXMin;
                NewX = cXScale(ax);
                
                ay=py[j];
                if(ay > rYMax)ay = rYMax; else if(ay < rYMin)ay = rYMin;
                NewY = cYScale(ay);
                
                if((NewX != OldX) || (NewY != OldY )){
                    OldX = NewX ;
                    OldY = NewY ;
                    glVertex2i(NewX, NewY);
                }
            }	
            glEnd();
        }
    }
}

//-----------------------------------
//       Mouse
//-----------------------------------


void CGraphView::mouseUp(int x0, int y0, int x1, int y1)
{
    if(x0 == x1) return;
    if(x0 > x1){ int tmp=x0; x0=x1, x1=tmp;}
    
    pGraph->XMin = cXScaleBack(x0);
    pGraph->XMax = cXScaleBack(x1);
    
    pGraph->findXRange(pGraph->XMin, pGraph->XMax);
    
}
void CGraphView::mouseUpRight(int x0, int y0, int x1, int y1)
{
    if(y0 == y1) return;
    if(y0 > y1){ int tmp=y0; y0=y1, y1=tmp;}
    pGraph->YMax = cYScaleBack(y0);
    pGraph->YMin = cYScaleBack(y1);    
}
void CGraphView::fullWaves() 
{
    pGraph->XPStart  = 0;
    pGraph->Points  = pGraph->npts;
    
    pGraph->XOrigin = pGraph->W[pGraph->XAxisNo].aMin;
    pGraph->XDelta  = (pGraph->W[pGraph->XAxisNo].aMax - pGraph->W[pGraph->XAxisNo].aMin)/(pGraph->npts-1);
    
    if(pGraph->XAxisNo==0){
        pGraph->XMin = pGraph->W[pGraph->XAxisNo].A[pGraph->XPStart];
        pGraph->XMax = pGraph->W[pGraph->XAxisNo].A[pGraph->XPStart + pGraph->Points - 1];
    }else{
        pGraph->XMin = pGraph->W[pGraph->XAxisNo].aMin;
        pGraph->XMax = pGraph->W[pGraph->XAxisNo].aMax;
    }
    
    pGraph->fullGraph();
    
}

void CGraphView::changeMode(int iaxis)
{
	pGraph->XAxisNo = iaxis;
	
    pGraph->XOrigin = pGraph->W[pGraph->XAxisNo].aMin;
    pGraph->XDelta  = (pGraph->W[pGraph->XAxisNo].aMax - pGraph->W[pGraph->XAxisNo].aMin)/(pGraph->npts-1);
    
    if(pGraph->XAxisNo==0){
        pGraph->XMin = pGraph->W[pGraph->XAxisNo].A[pGraph->XPStart];
        pGraph->XMax = pGraph->W[pGraph->XAxisNo].A[pGraph->XPStart + pGraph->Points - 1];
    }else{
        pGraph->XPStart = 0;
        pGraph->Points = pGraph->npts;
        pGraph->XMin = pGraph->W[pGraph->XAxisNo].aMin;
        pGraph->XMax = pGraph->W[pGraph->XAxisNo].aMax;
    }
}


